# gir2rst - Create Sphinx documentation from GIR files
# Copyright (C) 2012 Matthias Vogelgesang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import string
from xml.etree import ElementTree

NS_CORE_PREFIX = '{http://www.gtk.org/introspection/core/1.0}'
NS_C_PREFIX = '{http://www.gtk.org/introspection/c/1.0}'


class GirParser(object):
    def __init__(self, filename):
        self.tree = ElementTree.ElementTree()
        self.tree.parse(filename)
        self.ns_core = string.Template('%s$tag' % (NS_CORE_PREFIX))
        self.ns_c = string.Template('%s$tag' % (NS_C_PREFIX))

    def get_namespace(self):
        return self.tree.find(self.ns_core.substitute(tag='namespace'))

    def get_classes(self):
        ns_tag = self.ns_core.substitute(tag='namespace')
        class_tag = self.ns_core.substitute(tag='class')
        return self.tree.findall('%s/%s' % (ns_tag, class_tag))

    def get_class_name(self, class_element):
        return class_element.attrib['name']

    def get_c_type_attrib(self, element):
        if element is None:
            return None
        return element.attrib[self.ns_c.substitute(tag='type')]

    def get_type(self, element):
        return element.find(self.ns_core.substitute(tag='type'))

    def get_element_doc(self, element):
        return element.findtext(self.ns_core.substitute(tag='doc'))

    def get_constructor(self, class_element):
        return class_element.find(self.ns_core.substitute(tag='constructor'))

    def get_methods(self, class_element):
        return class_element.findall(self.ns_core.substitute(tag='method'))

    def get_parameters(self, method_element):
        parameters_tag = self.ns_core.substitute(tag='parameters')
        parameter_tag = self.ns_core.substitute(tag='parameter')
        return method_element.findall('%s/%s' % (parameters_tag,
            parameter_tag))

    def get_return_value(self, method_element):
        return method_element.find(self.ns_core.substitute(tag='return-value'))

    def get_return_type(self, method_element):
        rvalue = self.get_return_value(method_element)
        return rvalue.find(self.ns_core.substitute(tag='type'))



from distutils.core import setup

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Software Development :: Documentation",
    "Topic :: Text Processing :: Markup"
]

with open('README.rst') as fp:
    LONG_DESCRIPTION = fp.read()

setup(
    name='gir2rst', 
    version='0.2', 
    packages=['gir2rst'],
    scripts=['bin/gir2rst'],
    author='Matthias Vogelgesang',
    author_email='matthias.vogelgesang@gmail.com',
    url='http://github.com/matze/gir2rst',
    keywords=['rst', 'gir'],
    license='GPL v3',
    classifiers=CLASSIFIERS,
    description='ReST-based Sphinx documentation from GIR files',
    long_description=LONG_DESCRIPTION
)



# gir2rst - Create Sphinx documentation from GIR files
# Copyright (C) 2012 Matthias Vogelgesang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import string
import textwrap
import gir2rst.parser


def rst_h1(title, char='='):
    lines = char * len(title)
    return "%s\n%s\n%s\n\n" % (lines, title, lines)


def rst_h2(title, char='='):
    lines = char * len(title)
    return "%s\n%s\n\n" % (title, lines)


class RstFormatter(object):
    def __init__(self, filename, output):
        self.parser = gir2rst.parser.GirParser(filename)
        self.output = output
        self.wrapper = textwrap.TextWrapper(initial_indent='    ',
                subsequent_indent='    ')
        self.ns_core = string.Template('%s$tag' %
                gir2rst.parser.NS_CORE_PREFIX)
        self.ns_c = string.Template('%s$tag' % gir2rst.parser.NS_C_PREFIX)

    def write_rst(self):
        namespace = self.parser.get_namespace()
        title = "%s %s API reference" % (namespace.attrib['name'],
                namespace.attrib['version'])
        self.output.write(rst_h1(title))

        for class_element in self.parser.get_classes():
            self.output_class(class_element)
            constructor = self.parser.get_constructor(class_element)
            if constructor:
                self.output_constructor(constructor)

            for method in self.parser.get_methods(class_element):
                self.output_method(method)

    def print_lines(self, lines):
        if lines:
            for line in self.wrapper.wrap(lines):
                self.output.write(line + '\n')

    def output_class(self, class_element):
        pass

    def output_constructor(self, element, rtype):
        pass

    def output_method(self, element, rtype):
        pass


class RstCFormatter(RstFormatter):
    def __init__(self, filename, output):
        super(RstCFormatter, self).__init__(filename, output)

    def output_class(self, class_element):
        name = self.parser.get_c_type_attrib(class_element)
        self.output.write(rst_h2(name))
        self.output.write(".. c:type:: %s\n\n" % name)
        self.print_lines(self.parser.get_element_doc(class_element))
        self.output.write('\n\n')

    def output_constructor(self, element):
        self.output_method(element)

    def output_method(self, element):
        rtype = self.parser.get_return_type(element)
        c_name = element.attrib[self.ns_c.substitute(tag='identifier')]
        c_rtype = rtype.attrib[self.ns_c.substitute(tag='type')]
        self.output.write(".. c:function:: %s %s(" % (c_rtype, c_name))

        params = self.parser.get_parameters(element)
        param_names = [p.attrib['name'] for p in params]
        types = [self.parser.get_c_type_attrib(self.parser.get_type(param)) for
                param in params]

        tokens = [', '] * len(param_names)
        if tokens:
            tokens[-1] = ''

        zipped = zip(types, param_names, tokens)
        self.output.write("".join(["%s %s%s" % (a, b, c) for (a, b, c) in
            zipped]))
        self.output.write(")\n\n")

        self.print_lines(self.parser.get_element_doc(element))
        self.output.write('\n')

        param_descriptions = ["    :param %s: %s\n" % (p.attrib['name'],
            self.parser.get_element_doc(p)) for p in params]

        for desc in param_descriptions:
            self.output.write(desc)

        rval = self.parser.get_return_value(element)
        if self.parser.get_type(rval).attrib['name'] != 'none':
            self.output.write('\n    :returns: %s' %
                    self.parser.get_element_doc(rval))

        self.output.write('\n\n')
